%{
    Bagpipe music settings for Lilypond.
    This file builds on work by Andrew McNabb (http://www.mcnabbs.org/andrew/)
    
    Substantial changes and additions made by Sven Axelsson, MPD
    
    $Id: bagpipe.ly 77 2005-07-11 10:17:54Z sven $
%}

\version "2.6.0"

% No point-and-click here.

#(ly:set-option 'point-and-click #f)

% Hack for page breaking in a multi score book.

#(if (not (defined? 'doBreakBefore))
  (define doBreakBefore #f))

% Notes of the scale of the Great Highland Bagpipe:

pitchnamesBagpipe = #`(
	(G . ,(ly:make-pitch 0 4 NATURAL))
	(a . ,(ly:make-pitch 0 5 NATURAL))
	(b . ,(ly:make-pitch 0 6 NATURAL))
	(c . ,(ly:make-pitch 1 0 SHARP))
	(d . ,(ly:make-pitch 1 1 NATURAL))
	(e . ,(ly:make-pitch 1 2 NATURAL))
	(f . ,(ly:make-pitch 1 3 SHARP))
	(g . ,(ly:make-pitch 1 4 NATURAL))
	(A . ,(ly:make-pitch 1 5 NATURAL))
)
pitchnames = \pitchnamesBagpipe
#(ly:parser-set-note-names parser pitchnames)

% Some common timing tweaks

% Sets the autobeamer to span quarter notes only. Use for fast music
% Todo: Needs more tweaking
quarterBeaming = {
    #(override-auto-beam-setting '(end * * * *) 1 4 'Staff)
    #(override-auto-beam-setting '(end * * * *) 1 2 'Staff)
    #(override-auto-beam-setting '(end * * * *) 3 4 'Staff)
    #(override-auto-beam-setting '(end * * * *) 4 4 'Staff)
    #(revert-auto-beam-setting '(end 1 32 2 4 ) 1 8 'Staff)
    #(revert-auto-beam-setting '(end 1 32 2 4 ) 3 8 'Staff)
    #(revert-auto-beam-setting '(end 1 32 4 4 ) 1 8 'Staff)
    #(revert-auto-beam-setting '(end 1 32 4 4 ) 3 8 'Staff)
    #(revert-auto-beam-setting '(end 1 32 4 4 ) 5 8 'Staff)
    #(revert-auto-beam-setting '(end 1 32 4 4 ) 7 8 'Staff)
}
halfBeaming = {
    #(override-auto-beam-setting '(end * * 2 2) 1 2 'Staff)
    #(override-auto-beam-setting '(end * * 2 2) 2 2 'Staff)
}
% Reels with four beats to the bar in allabreve time with quarter beaming
% This probably doesn't exist, so don't use it :)
slowReelTime = {
    \time 2/2
    \set Score.measureLength = #(ly:make-moment 2 4)
    \quarterBeaming
}
% Reels are in allabreve time with quarter beaming
reelTime = {
    \time 2/2
    \halfBeaming
}
% 4/4 marches are written with numerical time signature and with quarter beaming
marchTime = {
    \time 4/4
    \override Staff.TimeSignature #'style = #'()
    \quarterBeaming
}

% Layout tweaks for bagpipe music

\layout {

    % No first line indent
    indent = 0.0 \cm
    
    \context {
        \Score
        % No bar numbers
        \remove "Bar_number_engraver"
        % Make sure the volta brackets are put at a fixed height
        \override VoltaBracket #'minimum-space = #7.2
        \override VoltaBracket #'height = #2.2
        % Copied from engraver-init.ly and tweaked a little to get smaller grace notes
        graceSettings = #`(
            (Voice Stem direction 1)            
            (Voice Stem lengths ,(map (lambda (x) (* 0.8 x)) '(3.5 3.5 3.5 4.5 5.0)))
            (Voice Stem stem-shorten (0.4 0.4))
            (Voice Stem font-size -4)
            (Voice NoteHead font-size -4)
            (Voice Dots font-size -4)
            (Voice Stem beamed-lengths  
             ,(map (lambda (x) (* 0.8 x)) '(3.3 3.3 4.0)))
            (Voice Stem beamed-minimum-free-lengths  
             ,(map (lambda (x) (* 0.8 x)) '(2.5 2.0 1.5)))
            (Voice Stem beamed-extreme-minimum-free-lengths  
             ,(map (lambda (x) (* 0.8 x)) '(1.83 1.5)))
        
            (Voice Stem no-stem-extend #t)
            (Voice Beam thickness 0.384)
            (Voice Beam space-function ,(lambda (beam mult)
                         (* 0.8 (Beam::space_function
                             beam mult))))
            (Voice Accidental font-size -4)
            (Voice Slur direction -1)
          )    
        }
        
	\context {
		\Staff
        % No key signature. N.b. this is not optimal, since removing the accidential
        % engraver means that we can't use c flat in the (few) places that is needed.
        \remove "Key_engraver"
        \remove "Accidental_engraver"
	}

	\context {
		\Voice
        % All stems go down
		\override Stem #'direction = #-1
        % All slurs and ties are on top
		\override Slur #'direction = #1
		\override Tie #'direction = #1
	}

}

% Single Grace Notes
grG = { \grace { G32 } }
gra = { \grace { a32 } }
grb = { \grace { b32 } }
grc = { \grace { c32 } }
grd = { \grace { d32 } }
gre = { \grace { e32 } }
grf = { \grace { f32 } }
grg = { \grace { g32 } }
grA = { \grace { A32 } }

% Doublings
dblG = { \grace { g32[ G d] } }
dbla = { \grace { g32[ a d] } }
dblb = { \grace { g32[ b d] } }
dblc = { \grace { g32[ c d] } }
dbld = { \grace { g32[ d e] } }
dble = { \grace { g32[ e f] } }
dblf = { \grace { g32[ f g] } }
% These are the same as the half doublings
dblg = { \grace { g32[ f] } }
dblA = { \grace { A32[ g] } }

% Half doublings
hdblG = { \grace { G32[ d] } }
hdbla = { \grace { a32[ d] } }
hdblb = { \grace { b32[ d] } }
hdblc = { \grace { c32[ d] } }
hdbld = { \grace { d32[ e] } }
hdble = { \grace { e32[ f] } }
hdblf = { \grace { f32[ g] } }
hdblg = { \grace { g32[ f] } }
hdblA = { \grace { A32[ g] } }

% Thumb doublings
tdblG = { \grace { A32[ G d] } }
tdbla = { \grace { A32[ a d] } }
tdblb = { \grace { A32[ b d] } }
tdblc = { \grace { A32[ c d] } }
tdbld = { \grace { A32[ d e] } }
tdble = { \grace { A32[ e f] } }
tdblf = { \grace { A32[ f g] } }
tdblg = { \grace { A32[ g f] } }

% Shakes
% A few of these can't really be played and are here only for consistency
shakea = { \grace { g32[ a d a G] } }
shakeb = { \grace { g32[ b d b G] } }
shakec = { \grace { g32[ c d c G] } }
shaked = { \grace { g32[ d e d G] } }
shakee = { \grace { g32[ e f e a] } }
shakef = { \grace { g32[ f g f a] } }
shakeg = { \grace { A32[ f g a] } }
shakeA = { \grace { A32[ g A a] } }

% Half shakes
hshakea = { \grace { a32[ d a G] } }
hshakeb = { \grace { b32[ d b G] } }
hshakec = { \grace { c32[ d c G] } }
hshaked = { \grace { d32[ e d G] } }
hshakee = { \grace { e32[ f e a] } }
hshakef = { \grace { f32[ g f a] } }
hshakeg = { \grace { g32[ f g a] } }
hshakeA = { \grace { A32[ g A a] } }

% Thumb shakes
tshakea = { \grace { A32[ a d a G] } }
tshakeb = { \grace { A32[ b d b G] } }
tshakec = { \grace { A32[ c d c G] } }
tshaked = { \grace { A32[ d e d G] } }
tshakee = { \grace { A32[ e f e a] } }
tshakef = { \grace { A32[ f g f a] } }
tshakeg = { \grace { A32[ f g a] } }
tshakeA = { \grace { A32[ g A a] } }

% Slurs
% A few of these can't really be played and are here only for consistency
slura = { \grace { g32[ a G] } }
slurb = { \grace { g32[ b G] } }
slurc = { \grace { g32[ c G] } }
slurd = { \grace { g32[ d G] } }
slure = { \grace { g32[ e a] } }
slurf = { \grace { g32[ f a] } }
slurg = { \grace { A32[ f a] } }
slurA = { \grace { f32[ a] } }

% Half slurs
hslura = { \grace { a32[ G] } }
hslurb = { \grace { b32[ G] } }
hslurc = { \grace { c32[ G] } }
hslurd = { \grace { d32[ G] } }
hslure = { \grace { e32[ a] } }
hslurf = { \grace { f32[ a] } }
hslurg = { \grace { g32[ a] } }
hslurA = { \grace { A32[ a] } }

% Thumb slurs
tslura = { \grace { A32[ a G] } }
tslurb = { \grace { A32[ b G] } }
tslurc = { \grace { A32[ c G] } }
tslurd = { \grace { A32[ d a] } }
tslure = { \grace { A32[ e a] } }
tslurf = { \grace { A32[ f a] } }
tslurg = { \grace { A32[ f a] } }
tslurA = { \grace { f32[ a] } }

% Throws
thrwd  = { \grace { G32[ d c] } }
Gthrwd = { \grace { d32[ c] } }
pthrwd = { \grace { G16[ d32 c] } }

% Birls
birl  = { \grace { a32[ G a G] } }
wbirl = { \grace { G32[ a G] } }
gbirl = { \grace { g32[ a G a G] } }
dbirl = { \grace { d32[ a G a G] } }
sbirl = gbirl

% Grips
grip  = { \grace { G32[ d G] } }
dgrip = { \grace { G32[ b G] } }
egrip = { \grace { G32[ e G] } }
gripb  = { \grace { b32[ G d G] } }
gripc  = { \grace { c32[ G d G] } }

% Taorluaths
taor    = { \grace { G32[ d G e] } }
dtaor   = { \grace { G32[ b G e] } }
Gtaor   = { \grace { d32[ G e] } }
taoramb = { \grace { G32[ d G b e] } }
taoramc = { \grace { G32[ d G c e] } }
taoramd = { \grace { G32[ d G c d e] } }

% Crunluaths
crun     = { \grace { G32[ d G e a f a ] } }
dcrun    = { \grace { G32[ b G e a f a ] } }
Gcrun    = { \grace { d32[ G e G f a ] } }
crunamb  = { \grace { G32[ d G b e b f b ] } }
crunamc  = { \grace { G32[ d G c e c f c ] } }
crunamd  = { \grace { G32[ d G c d e d f d ] } }

% Special piobaireachd notations
grGcad  = { \grace { G16 } }
gracad  = { \grace { a16 } }
cad     = { \grace { g32[ e8 d32] } }
hcad    = { \grace { g32[ e8] } }
edre    = { \grace { e32[ a f a] } }
embari  = { \grace { e32[ G f G] } }
chedari = { \grace { f32[ e g e f e] } }
